/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CPU_OPERATOR_BITSHIFTIMPL_KERNELS_H_
#define AIDGE_CPU_OPERATOR_BITSHIFTIMPL_KERNELS_H_

#include "aidge/utils/Registrar.hpp"

#include <cstdint>     // std::int32_t, std::int64_t
#include "aidge/operator/BitShift.hpp"

#include "aidge/backend/cpu/data/Broadcasting.hpp"
#include "aidge/backend/cpu/operator/BitShiftImpl.hpp"



namespace Aidge {
template <class I1, class I2, class O>
void BitShiftImpl_cpu_forward_kernel(
                                const BitShift_Op::BitShiftDirection direction,
                                const std::vector<std::size_t>& input1Dims,
                                const std::vector<std::size_t>& input2Dims,
                                const std::vector<std::size_t>& outputDims,
                                const void* input1_,
                                const void* input2_,
                                void* output_
                                ) {

    const I1* input_1 = static_cast<const I1*>(input1_);
    const I2* input_2 = static_cast<const I2*>(input2_);
    O* output = static_cast<O*>(output_);

    const size_t totalElements = std::accumulate(outputDims.begin(), outputDims.end(), std::size_t(1), std::multiplies<std::size_t>());
    
    for (std::size_t oIndex = 0; oIndex < totalElements; ++oIndex)
    {
        std::vector<size_t> indexes = getMultiDimIndices(outputDims, oIndex);
        std::size_t idx1 = getFlattenedIndex(input1Dims, indexes);
        std::size_t idx2 = getFlattenedIndex(input2Dims, indexes);
        if(direction == BitShift_Op::BitShiftDirection::right)

        {
                output[oIndex]= input_1[idx1] >> input_2[idx2];
        }
        else
        {
                output[oIndex] = input_1[idx1] << input_2[idx2];
        }
    }
}

REGISTRAR(BitShiftImpl_cpu,
{DataType::Int32},
{ProdConso::inPlaceModel,Aidge::BitShiftImpl_cpu_forward_kernel<std::int32_t, std::int32_t, std::int32_t>,nullptr});
REGISTRAR(BitShiftImpl_cpu,
{DataType::Int64},
{ProdConso::inPlaceModel,Aidge::BitShiftImpl_cpu_forward_kernel<std::int64_t, std::int64_t, std::int64_t>,nullptr});


}  // namespace Aidge

#endif /* AIDGE_CPU_OPERATOR_BitShiftIMPL_KERNELS_H_ */